package main

import (
	"flag"
	"fmt"
	"log"
	"myapp/internal/driver"
	"myapp/internal/models"
	"net/http"
	"os"
	"time"
)

const (
	version = "1.0.0"
)

type config struct {
	port int
	env  string
	db   struct {
		dsn string
	}
	stripe struct {
		secret string
		key    string
	}
	smtp struct {
		host     string
		port     int
		username string
		password string
	}
}

type application struct {
	config   config
	infoLog  *log.Logger
	errorLog *log.Logger
	version  string
	DB       models.DBModel
}

func (app *application) serve() error {
	srv := &http.Server{
		Addr:              fmt.Sprintf(":%d", app.config.port),
		Handler:           app.routes(),
		IdleTimeout:       30 * time.Second,
		ReadTimeout:       10 * time.Second,
		ReadHeaderTimeout: 5 * time.Second,
		WriteTimeout:      5 * time.Second,
	}

	app.infoLog.Printf(
		"Starting Backend server in %s mode on port %d",
		app.config.env,
		app.config.port,
	)
	return srv.ListenAndServe()
}

func main() {
	var cfg config

	flag.IntVar(&cfg.port, "port", 4001, "Server port to listen on")
	flag.StringVar(
		&cfg.env,
		"env",
		"development",
		"Application environment {development|production|maintenance}",
	)
	flag.StringVar(
		&cfg.db.dsn,
		"dsn",
		"root:example@tcp(localhost:3306)/widgets?parseTime=true&tls=false",
		"Application environment {development|production}",
	)
	flag.StringVar(&cfg.smtp.host, "smtphost", "0.0.0.0", "smtp host")
	flag.IntVar(&cfg.smtp.port, "smtpport", 1025, "smtp host")
	flag.StringVar(&cfg.smtp.username, "smtpuser", "user", "smtp user")
	flag.StringVar(&cfg.smtp.password, "smtppwd", "password", "smtp password")

	flag.Parse()

	cfg.stripe.key = os.Getenv("STRIPE_KEY")
	cfg.stripe.secret = os.Getenv("STRIPE_SECRET")

	infoLog := log.New(os.Stdout, "INFO\t", log.Ldate|log.Ltime)
	errorLog := log.New(os.Stdout, "ERROR\t", log.Ldate|log.Ltime|log.Lshortfile)

	conn, err := driver.OpenDB(cfg.db.dsn)
	if err != nil {
		errorLog.Fatal(err)
	}
	defer conn.Close()

	app := &application{
		version:  version,
		config:   cfg,
		infoLog:  infoLog,
		errorLog: errorLog,
		DB:       models.DBModel{DB: conn},
	}

	app.infoLog.Println("Connected to MariaDB")

	err = app.serve()
	if err != nil {
		log.Fatal(err)
	}
}
